/**
 * Clone Document
 *
 * Clone current document into new Tab.
 *
 * @category  WeBuilder Plugin
 * @package   Clone document
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2018
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.01
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     31   function CloneDocument()
 *    114   function ReturnToOriginalTab(fileName)
 *    128   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 3
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

/**
 * Clone current document into new Tab
 *
 * @return void
 */
function CloneDocument() {

	// Save info of original document, such as path/name, encoding, selection and scroll position.
	var filename = Document.FileName;
	var ext = ExtractFileExt(filename);
	var encoding = Document.Encoding;
	var content = Editor.Text;
	var uSel = Editor.Selection;
	var lineTop = Editor.TopLine;
	var startLine = uSel.SelStartLine;
	var endLine = uSel.SelEndLine;
	var startColumn = uSel.SelStartCol;
	var endColumn = uSel.SelEndCol;

	// If ext is missing, then this must be a new unsaved document.
	// So extension must be based on document type instead.
	if (ext == "") {
		switch (Document.DocType) {
			case dtHTML:		ext = ".html";
			case dtXML:			ext = ".xml";
			case dtCSS:			ext = ".css";
			case dtJScript:		ext = ".js";
			case dtVBScript:	ext = ".vbs";
			case dtPerl:		ext = ".pl";
			case dtText:		ext = ".txt";
			case dtWML:	   		ext = ".wml";
			case dtPHP:	   		ext = ".php";
			case dtASP:	   		ext = ".asp";
			case dtSQL:			ext = ".sql";
			case dtRuby:		ext = ".rb";
			case dtERuby:		ext = ".rhtml";
			case dtASPX:		ext = ".aspx";
			case dtCS:	   		ext = ".cs";
			case dtSmarty:		ext = ".tpl";
			case dtPython:		ext = ".py";
			case dtApache:		ext = ".htaccess";
			case dtSass:		ext = ".scss";
			case dtLess:		ext = ".less";
		}
	}

	// Create clone document
	Documents.NewDocument(ext);
	Document.Encoding = encoding;

	// New unsaved documents doesn't have a filename.
	if (filename != "") {
		if (Script.ReadSetting("Cloned path/filename equal to original", "0") == "1") {
			// Full path/name (Cloned document is identical to original)
			Document.Filename = filename;
		}
		else {
			// Just the name (Document can ONLY be saved through the "Save As" dialog)
			Document.Filename = ExtractFileName(filename);
		}
	}

	// Clone document content, selection and scroll position.
	Editor.Text = content;
	Editor.TopLine = lineTop;
	uSel.SelStartLine = startLine;
	uSel.SelStartCol = startColumn;
	uSel.SelEndLine = endLine;
	uSel.SelEndCol = endColumn;
	Editor.Selection = uSel;

	if (Script.ReadSetting("Mark cloned document as modified", "1") == "0") {
		Editor.Modified = false;
	}

	if (Script.ReadSetting("Return to original Editor Tab after cloning", "1") == "1") {
		ReturnToOriginalTab(fileName);
	}

}

/**
 * Return to original Editor Tab after opening new Document/Tab
 *
 * @param  string   fileName path/name of file in original Tab
 *
 * @return void
 */
function ReturnToOriginalTab(fileName) {
	for (var i=0;i<Documents.Count;i++) {
		if (Documents.Tab[i].Filename == fileName) {
			Documents.Tab[i].Activate;
			break;
		}
	}
}

/**
 * Show info when plugin is installed
 *
 * @return void
 */
function OnInstalled() {
	alert("Clone Document 1.01 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");

var bmp = new TBitmap, act;
LoadFileToBitmap(Script.Path + "document_copies.png", bmp);
act = Script.RegisterDocumentAction("", "Clone document", "", "CloneDocument");
Actions.SetIcon(act, bmp);
delete bmp;